r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Api
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page
from twilio.rest.api.v2010.account.sip.domain.auth_types import AuthTypesList
from twilio.rest.api.v2010.account.sip.domain.credential_list_mapping import (
    CredentialListMappingList,
)
from twilio.rest.api.v2010.account.sip.domain.ip_access_control_list_mapping import (
    IpAccessControlListMappingList,
)


class DomainInstance(InstanceResource):
    def __init__(self, version, payload, account_sid: str, sid: Optional[str] = None):
        """
        Initialize the DomainInstance

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        super().__init__(version)

        self._properties = {
            "account_sid": payload.get("account_sid"),
            "api_version": payload.get("api_version"),
            "auth_type": payload.get("auth_type"),
            "date_created": deserialize.rfc2822_datetime(payload.get("date_created")),
            "date_updated": deserialize.rfc2822_datetime(payload.get("date_updated")),
            "domain_name": payload.get("domain_name"),
            "friendly_name": payload.get("friendly_name"),
            "sid": payload.get("sid"),
            "uri": payload.get("uri"),
            "voice_fallback_method": payload.get("voice_fallback_method"),
            "voice_fallback_url": payload.get("voice_fallback_url"),
            "voice_method": payload.get("voice_method"),
            "voice_status_callback_method": payload.get("voice_status_callback_method"),
            "voice_status_callback_url": payload.get("voice_status_callback_url"),
            "voice_url": payload.get("voice_url"),
            "subresource_uris": payload.get("subresource_uris"),
            "sip_registration": payload.get("sip_registration"),
            "emergency_calling_enabled": payload.get("emergency_calling_enabled"),
            "secure": payload.get("secure"),
            "byoc_trunk_sid": payload.get("byoc_trunk_sid"),
            "emergency_caller_sid": payload.get("emergency_caller_sid"),
        }

        self._solution = {
            "account_sid": account_sid,
            "sid": sid or self._properties["sid"],
        }
        self._context: Optional[DomainContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: DomainContext for this DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainContext
        """
        if self._context is None:
            self._context = DomainContext(
                self._version,
                account_sid=self._solution["account_sid"],
                sid=self._solution["sid"],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the SipDomain resource.
        :rtype: str
        """
        return self._properties["account_sid"]

    @property
    def api_version(self):
        """
        :returns: The API version used to process the call.
        :rtype: str
        """
        return self._properties["api_version"]

    @property
    def auth_type(self):
        """
        :returns: The types of authentication you have mapped to your domain. Can be: `IP_ACL` and `CREDENTIAL_LIST`. If you have both defined for your domain, both will be returned in a comma delimited string. If `auth_type` is not defined, the domain will not be able to receive any traffic.
        :rtype: str
        """
        return self._properties["auth_type"]

    @property
    def date_created(self):
        """
        :returns: The date and time in GMT that the resource was created specified in [RFC 2822](https://www.ietf.org/rfc/rfc2822.txt) format.
        :rtype: datetime
        """
        return self._properties["date_created"]

    @property
    def date_updated(self):
        """
        :returns: The date and time in GMT that the resource was last updated specified in [RFC 2822](https://www.ietf.org/rfc/rfc2822.txt) format.
        :rtype: datetime
        """
        return self._properties["date_updated"]

    @property
    def domain_name(self):
        """
        :returns: The unique address you reserve on Twilio to which you route your SIP traffic. Domain names can contain letters, digits, and \"-\" and must end with `sip.twilio.com`.
        :rtype: str
        """
        return self._properties["domain_name"]

    @property
    def friendly_name(self):
        """
        :returns: The string that you assigned to describe the resource.
        :rtype: str
        """
        return self._properties["friendly_name"]

    @property
    def sid(self):
        """
        :returns: The unique string that that we created to identify the SipDomain resource.
        :rtype: str
        """
        return self._properties["sid"]

    @property
    def uri(self):
        """
        :returns: The URI of the resource, relative to `https://api.twilio.com`.
        :rtype: str
        """
        return self._properties["uri"]

    @property
    def voice_fallback_method(self):
        """
        :returns: The HTTP method we use to call `voice_fallback_url`. Can be: `GET` or `POST`.
        :rtype: str
        """
        return self._properties["voice_fallback_method"]

    @property
    def voice_fallback_url(self):
        """
        :returns: The URL that we call when an error occurs while retrieving or executing the TwiML requested from `voice_url`.
        :rtype: str
        """
        return self._properties["voice_fallback_url"]

    @property
    def voice_method(self):
        """
        :returns: The HTTP method we use to call `voice_url`. Can be: `GET` or `POST`.
        :rtype: str
        """
        return self._properties["voice_method"]

    @property
    def voice_status_callback_method(self):
        """
        :returns: The HTTP method we use to call `voice_status_callback_url`. Either `GET` or `POST`.
        :rtype: str
        """
        return self._properties["voice_status_callback_method"]

    @property
    def voice_status_callback_url(self):
        """
        :returns: The URL that we call to pass status parameters (such as call ended) to your application.
        :rtype: str
        """
        return self._properties["voice_status_callback_url"]

    @property
    def voice_url(self):
        """
        :returns: The URL we call using the `voice_method` when the domain receives a call.
        :rtype: str
        """
        return self._properties["voice_url"]

    @property
    def subresource_uris(self):
        """
        :returns: A list of mapping resources associated with the SIP Domain resource identified by their relative URIs.
        :rtype: dict
        """
        return self._properties["subresource_uris"]

    @property
    def sip_registration(self):
        """
        :returns: Whether to allow SIP Endpoints to register with the domain to receive calls.
        :rtype: bool
        """
        return self._properties["sip_registration"]

    @property
    def emergency_calling_enabled(self):
        """
        :returns: Whether emergency calling is enabled for the domain. If enabled, allows emergency calls on the domain from phone numbers with validated addresses.
        :rtype: bool
        """
        return self._properties["emergency_calling_enabled"]

    @property
    def secure(self):
        """
        :returns: Whether secure SIP is enabled for the domain. If enabled, TLS will be enforced and SRTP will be negotiated on all incoming calls to this sip domain.
        :rtype: bool
        """
        return self._properties["secure"]

    @property
    def byoc_trunk_sid(self):
        """
        :returns: The SID of the BYOC Trunk(Bring Your Own Carrier) resource that the Sip Domain will be associated with.
        :rtype: str
        """
        return self._properties["byoc_trunk_sid"]

    @property
    def emergency_caller_sid(self):
        """
        :returns: Whether an emergency caller sid is configured for the domain. If present, this phone number will be used as the callback for the emergency call.
        :rtype: str
        """
        return self._properties["emergency_caller_sid"]

    def delete(self):
        """
        Deletes the DomainInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    async def delete_async(self):
        """
        Asynchronous coroutine that deletes the DomainInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return await self._proxy.delete_async()

    def fetch(self):
        """
        Fetch the DomainInstance


        :returns: The fetched DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the DomainInstance


        :returns: The fetched DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        return await self._proxy.fetch_async()

    def update(
        self,
        friendly_name=values.unset,
        voice_fallback_method=values.unset,
        voice_fallback_url=values.unset,
        voice_method=values.unset,
        voice_status_callback_method=values.unset,
        voice_status_callback_url=values.unset,
        voice_url=values.unset,
        sip_registration=values.unset,
        domain_name=values.unset,
        emergency_calling_enabled=values.unset,
        secure=values.unset,
        byoc_trunk_sid=values.unset,
        emergency_caller_sid=values.unset,
    ):
        """
        Update the DomainInstance

        :param str friendly_name: A descriptive string that you created to describe the resource. It can be up to 64 characters long.
        :param str voice_fallback_method: The HTTP method we should use to call `voice_fallback_url`. Can be: `GET` or `POST`.
        :param str voice_fallback_url: The URL that we should call when an error occurs while retrieving or executing the TwiML requested by `voice_url`.
        :param str voice_method: The HTTP method we should use to call `voice_url`
        :param str voice_status_callback_method: The HTTP method we should use to call `voice_status_callback_url`. Can be: `GET` or `POST`.
        :param str voice_status_callback_url: The URL that we should call to pass status parameters (such as call ended) to your application.
        :param str voice_url: The URL we should call when the domain receives a call.
        :param bool sip_registration: Whether to allow SIP Endpoints to register with the domain to receive calls. Can be `true` or `false`. `true` allows SIP Endpoints to register with the domain to receive calls, `false` does not.
        :param str domain_name: The unique address you reserve on Twilio to which you route your SIP traffic. Domain names can contain letters, digits, and \\\"-\\\" and must end with `sip.twilio.com`.
        :param bool emergency_calling_enabled: Whether emergency calling is enabled for the domain. If enabled, allows emergency calls on the domain from phone numbers with validated addresses.
        :param bool secure: Whether secure SIP is enabled for the domain. If enabled, TLS will be enforced and SRTP will be negotiated on all incoming calls to this sip domain.
        :param str byoc_trunk_sid: The SID of the BYOC Trunk(Bring Your Own Carrier) resource that the Sip Domain will be associated with.
        :param str emergency_caller_sid: Whether an emergency caller sid is configured for the domain. If present, this phone number will be used as the callback for the emergency call.

        :returns: The updated DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        return self._proxy.update(
            friendly_name=friendly_name,
            voice_fallback_method=voice_fallback_method,
            voice_fallback_url=voice_fallback_url,
            voice_method=voice_method,
            voice_status_callback_method=voice_status_callback_method,
            voice_status_callback_url=voice_status_callback_url,
            voice_url=voice_url,
            sip_registration=sip_registration,
            domain_name=domain_name,
            emergency_calling_enabled=emergency_calling_enabled,
            secure=secure,
            byoc_trunk_sid=byoc_trunk_sid,
            emergency_caller_sid=emergency_caller_sid,
        )

    async def update_async(
        self,
        friendly_name=values.unset,
        voice_fallback_method=values.unset,
        voice_fallback_url=values.unset,
        voice_method=values.unset,
        voice_status_callback_method=values.unset,
        voice_status_callback_url=values.unset,
        voice_url=values.unset,
        sip_registration=values.unset,
        domain_name=values.unset,
        emergency_calling_enabled=values.unset,
        secure=values.unset,
        byoc_trunk_sid=values.unset,
        emergency_caller_sid=values.unset,
    ):
        """
        Asynchronous coroutine to update the DomainInstance

        :param str friendly_name: A descriptive string that you created to describe the resource. It can be up to 64 characters long.
        :param str voice_fallback_method: The HTTP method we should use to call `voice_fallback_url`. Can be: `GET` or `POST`.
        :param str voice_fallback_url: The URL that we should call when an error occurs while retrieving or executing the TwiML requested by `voice_url`.
        :param str voice_method: The HTTP method we should use to call `voice_url`
        :param str voice_status_callback_method: The HTTP method we should use to call `voice_status_callback_url`. Can be: `GET` or `POST`.
        :param str voice_status_callback_url: The URL that we should call to pass status parameters (such as call ended) to your application.
        :param str voice_url: The URL we should call when the domain receives a call.
        :param bool sip_registration: Whether to allow SIP Endpoints to register with the domain to receive calls. Can be `true` or `false`. `true` allows SIP Endpoints to register with the domain to receive calls, `false` does not.
        :param str domain_name: The unique address you reserve on Twilio to which you route your SIP traffic. Domain names can contain letters, digits, and \\\"-\\\" and must end with `sip.twilio.com`.
        :param bool emergency_calling_enabled: Whether emergency calling is enabled for the domain. If enabled, allows emergency calls on the domain from phone numbers with validated addresses.
        :param bool secure: Whether secure SIP is enabled for the domain. If enabled, TLS will be enforced and SRTP will be negotiated on all incoming calls to this sip domain.
        :param str byoc_trunk_sid: The SID of the BYOC Trunk(Bring Your Own Carrier) resource that the Sip Domain will be associated with.
        :param str emergency_caller_sid: Whether an emergency caller sid is configured for the domain. If present, this phone number will be used as the callback for the emergency call.

        :returns: The updated DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        return await self._proxy.update_async(
            friendly_name=friendly_name,
            voice_fallback_method=voice_fallback_method,
            voice_fallback_url=voice_fallback_url,
            voice_method=voice_method,
            voice_status_callback_method=voice_status_callback_method,
            voice_status_callback_url=voice_status_callback_url,
            voice_url=voice_url,
            sip_registration=sip_registration,
            domain_name=domain_name,
            emergency_calling_enabled=emergency_calling_enabled,
            secure=secure,
            byoc_trunk_sid=byoc_trunk_sid,
            emergency_caller_sid=emergency_caller_sid,
        )

    @property
    def auth(self):
        """
        Access the auth

        :returns: twilio.rest.api.v2010.account.sip.domain.AuthTypesList
        :rtype: twilio.rest.api.v2010.account.sip.domain.AuthTypesList
        """
        return self._proxy.auth

    @property
    def credential_list_mappings(self):
        """
        Access the credential_list_mappings

        :returns: twilio.rest.api.v2010.account.sip.domain.CredentialListMappingList
        :rtype: twilio.rest.api.v2010.account.sip.domain.CredentialListMappingList
        """
        return self._proxy.credential_list_mappings

    @property
    def ip_access_control_list_mappings(self):
        """
        Access the ip_access_control_list_mappings

        :returns: twilio.rest.api.v2010.account.sip.domain.IpAccessControlListMappingList
        :rtype: twilio.rest.api.v2010.account.sip.domain.IpAccessControlListMappingList
        """
        return self._proxy.ip_access_control_list_mappings

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Api.V2010.DomainInstance {}>".format(context)


class DomainContext(InstanceContext):
    def __init__(self, version: Version, account_sid: str, sid: str):
        """
        Initialize the DomainContext

        :param Version version: Version that contains the resource
        :param account_sid: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the SipDomain resource to update.
        :param sid: The Twilio-provided string that uniquely identifies the SipDomain resource to update.

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainContext
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "account_sid": account_sid,
            "sid": sid,
        }
        self._uri = "/Accounts/{account_sid}/SIP/Domains/{sid}.json".format(
            **self._solution
        )

        self._auth: Optional[AuthTypesList] = None
        self._credential_list_mappings: Optional[CredentialListMappingList] = None
        self._ip_access_control_list_mappings: Optional[
            IpAccessControlListMappingList
        ] = None

    def delete(self):
        """
        Deletes the DomainInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(
            method="DELETE",
            uri=self._uri,
        )

    async def delete_async(self):
        """
        Asynchronous coroutine that deletes the DomainInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return await self._version.delete_async(
            method="DELETE",
            uri=self._uri,
        )

    def fetch(self):
        """
        Fetch the DomainInstance


        :returns: The fetched DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return DomainInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            sid=self._solution["sid"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the DomainInstance


        :returns: The fetched DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return DomainInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            sid=self._solution["sid"],
        )

    def update(
        self,
        friendly_name=values.unset,
        voice_fallback_method=values.unset,
        voice_fallback_url=values.unset,
        voice_method=values.unset,
        voice_status_callback_method=values.unset,
        voice_status_callback_url=values.unset,
        voice_url=values.unset,
        sip_registration=values.unset,
        domain_name=values.unset,
        emergency_calling_enabled=values.unset,
        secure=values.unset,
        byoc_trunk_sid=values.unset,
        emergency_caller_sid=values.unset,
    ):
        """
        Update the DomainInstance

        :param str friendly_name: A descriptive string that you created to describe the resource. It can be up to 64 characters long.
        :param str voice_fallback_method: The HTTP method we should use to call `voice_fallback_url`. Can be: `GET` or `POST`.
        :param str voice_fallback_url: The URL that we should call when an error occurs while retrieving or executing the TwiML requested by `voice_url`.
        :param str voice_method: The HTTP method we should use to call `voice_url`
        :param str voice_status_callback_method: The HTTP method we should use to call `voice_status_callback_url`. Can be: `GET` or `POST`.
        :param str voice_status_callback_url: The URL that we should call to pass status parameters (such as call ended) to your application.
        :param str voice_url: The URL we should call when the domain receives a call.
        :param bool sip_registration: Whether to allow SIP Endpoints to register with the domain to receive calls. Can be `true` or `false`. `true` allows SIP Endpoints to register with the domain to receive calls, `false` does not.
        :param str domain_name: The unique address you reserve on Twilio to which you route your SIP traffic. Domain names can contain letters, digits, and \\\"-\\\" and must end with `sip.twilio.com`.
        :param bool emergency_calling_enabled: Whether emergency calling is enabled for the domain. If enabled, allows emergency calls on the domain from phone numbers with validated addresses.
        :param bool secure: Whether secure SIP is enabled for the domain. If enabled, TLS will be enforced and SRTP will be negotiated on all incoming calls to this sip domain.
        :param str byoc_trunk_sid: The SID of the BYOC Trunk(Bring Your Own Carrier) resource that the Sip Domain will be associated with.
        :param str emergency_caller_sid: Whether an emergency caller sid is configured for the domain. If present, this phone number will be used as the callback for the emergency call.

        :returns: The updated DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        data = values.of(
            {
                "FriendlyName": friendly_name,
                "VoiceFallbackMethod": voice_fallback_method,
                "VoiceFallbackUrl": voice_fallback_url,
                "VoiceMethod": voice_method,
                "VoiceStatusCallbackMethod": voice_status_callback_method,
                "VoiceStatusCallbackUrl": voice_status_callback_url,
                "VoiceUrl": voice_url,
                "SipRegistration": sip_registration,
                "DomainName": domain_name,
                "EmergencyCallingEnabled": emergency_calling_enabled,
                "Secure": secure,
                "ByocTrunkSid": byoc_trunk_sid,
                "EmergencyCallerSid": emergency_caller_sid,
            }
        )

        payload = self._version.update(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return DomainInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            sid=self._solution["sid"],
        )

    async def update_async(
        self,
        friendly_name=values.unset,
        voice_fallback_method=values.unset,
        voice_fallback_url=values.unset,
        voice_method=values.unset,
        voice_status_callback_method=values.unset,
        voice_status_callback_url=values.unset,
        voice_url=values.unset,
        sip_registration=values.unset,
        domain_name=values.unset,
        emergency_calling_enabled=values.unset,
        secure=values.unset,
        byoc_trunk_sid=values.unset,
        emergency_caller_sid=values.unset,
    ):
        """
        Asynchronous coroutine to update the DomainInstance

        :param str friendly_name: A descriptive string that you created to describe the resource. It can be up to 64 characters long.
        :param str voice_fallback_method: The HTTP method we should use to call `voice_fallback_url`. Can be: `GET` or `POST`.
        :param str voice_fallback_url: The URL that we should call when an error occurs while retrieving or executing the TwiML requested by `voice_url`.
        :param str voice_method: The HTTP method we should use to call `voice_url`
        :param str voice_status_callback_method: The HTTP method we should use to call `voice_status_callback_url`. Can be: `GET` or `POST`.
        :param str voice_status_callback_url: The URL that we should call to pass status parameters (such as call ended) to your application.
        :param str voice_url: The URL we should call when the domain receives a call.
        :param bool sip_registration: Whether to allow SIP Endpoints to register with the domain to receive calls. Can be `true` or `false`. `true` allows SIP Endpoints to register with the domain to receive calls, `false` does not.
        :param str domain_name: The unique address you reserve on Twilio to which you route your SIP traffic. Domain names can contain letters, digits, and \\\"-\\\" and must end with `sip.twilio.com`.
        :param bool emergency_calling_enabled: Whether emergency calling is enabled for the domain. If enabled, allows emergency calls on the domain from phone numbers with validated addresses.
        :param bool secure: Whether secure SIP is enabled for the domain. If enabled, TLS will be enforced and SRTP will be negotiated on all incoming calls to this sip domain.
        :param str byoc_trunk_sid: The SID of the BYOC Trunk(Bring Your Own Carrier) resource that the Sip Domain will be associated with.
        :param str emergency_caller_sid: Whether an emergency caller sid is configured for the domain. If present, this phone number will be used as the callback for the emergency call.

        :returns: The updated DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        data = values.of(
            {
                "FriendlyName": friendly_name,
                "VoiceFallbackMethod": voice_fallback_method,
                "VoiceFallbackUrl": voice_fallback_url,
                "VoiceMethod": voice_method,
                "VoiceStatusCallbackMethod": voice_status_callback_method,
                "VoiceStatusCallbackUrl": voice_status_callback_url,
                "VoiceUrl": voice_url,
                "SipRegistration": sip_registration,
                "DomainName": domain_name,
                "EmergencyCallingEnabled": emergency_calling_enabled,
                "Secure": secure,
                "ByocTrunkSid": byoc_trunk_sid,
                "EmergencyCallerSid": emergency_caller_sid,
            }
        )

        payload = await self._version.update_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return DomainInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            sid=self._solution["sid"],
        )

    @property
    def auth(self):
        """
        Access the auth

        :returns: twilio.rest.api.v2010.account.sip.domain.AuthTypesList
        :rtype: twilio.rest.api.v2010.account.sip.domain.AuthTypesList
        """
        if self._auth is None:
            self._auth = AuthTypesList(
                self._version,
                self._solution["account_sid"],
                self._solution["sid"],
            )
        return self._auth

    @property
    def credential_list_mappings(self):
        """
        Access the credential_list_mappings

        :returns: twilio.rest.api.v2010.account.sip.domain.CredentialListMappingList
        :rtype: twilio.rest.api.v2010.account.sip.domain.CredentialListMappingList
        """
        if self._credential_list_mappings is None:
            self._credential_list_mappings = CredentialListMappingList(
                self._version,
                self._solution["account_sid"],
                self._solution["sid"],
            )
        return self._credential_list_mappings

    @property
    def ip_access_control_list_mappings(self):
        """
        Access the ip_access_control_list_mappings

        :returns: twilio.rest.api.v2010.account.sip.domain.IpAccessControlListMappingList
        :rtype: twilio.rest.api.v2010.account.sip.domain.IpAccessControlListMappingList
        """
        if self._ip_access_control_list_mappings is None:
            self._ip_access_control_list_mappings = IpAccessControlListMappingList(
                self._version,
                self._solution["account_sid"],
                self._solution["sid"],
            )
        return self._ip_access_control_list_mappings

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Api.V2010.DomainContext {}>".format(context)


class DomainPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of DomainInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        return DomainInstance(
            self._version, payload, account_sid=self._solution["account_sid"]
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Api.V2010.DomainPage>"


class DomainList(ListResource):
    def __init__(self, version: Version, account_sid: str):
        """
        Initialize the DomainList

        :param Version version: Version that contains the resource
        :param account_sid: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the SipDomain resources to read.

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainList
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainList
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "account_sid": account_sid,
        }
        self._uri = "/Accounts/{account_sid}/SIP/Domains.json".format(**self._solution)

    def create(
        self,
        domain_name,
        friendly_name=values.unset,
        voice_url=values.unset,
        voice_method=values.unset,
        voice_fallback_url=values.unset,
        voice_fallback_method=values.unset,
        voice_status_callback_url=values.unset,
        voice_status_callback_method=values.unset,
        sip_registration=values.unset,
        emergency_calling_enabled=values.unset,
        secure=values.unset,
        byoc_trunk_sid=values.unset,
        emergency_caller_sid=values.unset,
    ):
        """
        Create the DomainInstance

        :param str domain_name: The unique address you reserve on Twilio to which you route your SIP traffic. Domain names can contain letters, digits, and \\\"-\\\" and must end with `sip.twilio.com`.
        :param str friendly_name: A descriptive string that you created to describe the resource. It can be up to 64 characters long.
        :param str voice_url: The URL we should when the domain receives a call.
        :param str voice_method: The HTTP method we should use to call `voice_url`. Can be: `GET` or `POST`.
        :param str voice_fallback_url: The URL that we should call when an error occurs while retrieving or executing the TwiML from `voice_url`.
        :param str voice_fallback_method: The HTTP method we should use to call `voice_fallback_url`. Can be: `GET` or `POST`.
        :param str voice_status_callback_url: The URL that we should call to pass status parameters (such as call ended) to your application.
        :param str voice_status_callback_method: The HTTP method we should use to call `voice_status_callback_url`. Can be: `GET` or `POST`.
        :param bool sip_registration: Whether to allow SIP Endpoints to register with the domain to receive calls. Can be `true` or `false`. `true` allows SIP Endpoints to register with the domain to receive calls, `false` does not.
        :param bool emergency_calling_enabled: Whether emergency calling is enabled for the domain. If enabled, allows emergency calls on the domain from phone numbers with validated addresses.
        :param bool secure: Whether secure SIP is enabled for the domain. If enabled, TLS will be enforced and SRTP will be negotiated on all incoming calls to this sip domain.
        :param str byoc_trunk_sid: The SID of the BYOC Trunk(Bring Your Own Carrier) resource that the Sip Domain will be associated with.
        :param str emergency_caller_sid: Whether an emergency caller sid is configured for the domain. If present, this phone number will be used as the callback for the emergency call.

        :returns: The created DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        data = values.of(
            {
                "DomainName": domain_name,
                "FriendlyName": friendly_name,
                "VoiceUrl": voice_url,
                "VoiceMethod": voice_method,
                "VoiceFallbackUrl": voice_fallback_url,
                "VoiceFallbackMethod": voice_fallback_method,
                "VoiceStatusCallbackUrl": voice_status_callback_url,
                "VoiceStatusCallbackMethod": voice_status_callback_method,
                "SipRegistration": sip_registration,
                "EmergencyCallingEnabled": emergency_calling_enabled,
                "Secure": secure,
                "ByocTrunkSid": byoc_trunk_sid,
                "EmergencyCallerSid": emergency_caller_sid,
            }
        )

        payload = self._version.create(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return DomainInstance(
            self._version, payload, account_sid=self._solution["account_sid"]
        )

    async def create_async(
        self,
        domain_name,
        friendly_name=values.unset,
        voice_url=values.unset,
        voice_method=values.unset,
        voice_fallback_url=values.unset,
        voice_fallback_method=values.unset,
        voice_status_callback_url=values.unset,
        voice_status_callback_method=values.unset,
        sip_registration=values.unset,
        emergency_calling_enabled=values.unset,
        secure=values.unset,
        byoc_trunk_sid=values.unset,
        emergency_caller_sid=values.unset,
    ):
        """
        Asynchronously create the DomainInstance

        :param str domain_name: The unique address you reserve on Twilio to which you route your SIP traffic. Domain names can contain letters, digits, and \\\"-\\\" and must end with `sip.twilio.com`.
        :param str friendly_name: A descriptive string that you created to describe the resource. It can be up to 64 characters long.
        :param str voice_url: The URL we should when the domain receives a call.
        :param str voice_method: The HTTP method we should use to call `voice_url`. Can be: `GET` or `POST`.
        :param str voice_fallback_url: The URL that we should call when an error occurs while retrieving or executing the TwiML from `voice_url`.
        :param str voice_fallback_method: The HTTP method we should use to call `voice_fallback_url`. Can be: `GET` or `POST`.
        :param str voice_status_callback_url: The URL that we should call to pass status parameters (such as call ended) to your application.
        :param str voice_status_callback_method: The HTTP method we should use to call `voice_status_callback_url`. Can be: `GET` or `POST`.
        :param bool sip_registration: Whether to allow SIP Endpoints to register with the domain to receive calls. Can be `true` or `false`. `true` allows SIP Endpoints to register with the domain to receive calls, `false` does not.
        :param bool emergency_calling_enabled: Whether emergency calling is enabled for the domain. If enabled, allows emergency calls on the domain from phone numbers with validated addresses.
        :param bool secure: Whether secure SIP is enabled for the domain. If enabled, TLS will be enforced and SRTP will be negotiated on all incoming calls to this sip domain.
        :param str byoc_trunk_sid: The SID of the BYOC Trunk(Bring Your Own Carrier) resource that the Sip Domain will be associated with.
        :param str emergency_caller_sid: Whether an emergency caller sid is configured for the domain. If present, this phone number will be used as the callback for the emergency call.

        :returns: The created DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainInstance
        """
        data = values.of(
            {
                "DomainName": domain_name,
                "FriendlyName": friendly_name,
                "VoiceUrl": voice_url,
                "VoiceMethod": voice_method,
                "VoiceFallbackUrl": voice_fallback_url,
                "VoiceFallbackMethod": voice_fallback_method,
                "VoiceStatusCallbackUrl": voice_status_callback_url,
                "VoiceStatusCallbackMethod": voice_status_callback_method,
                "SipRegistration": sip_registration,
                "EmergencyCallingEnabled": emergency_calling_enabled,
                "Secure": secure,
                "ByocTrunkSid": byoc_trunk_sid,
                "EmergencyCallerSid": emergency_caller_sid,
            }
        )

        payload = await self._version.create_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return DomainInstance(
            self._version, payload, account_sid=self._solution["account_sid"]
        )

    def stream(self, limit=None, page_size=None):
        """
        Streams DomainInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.api.v2010.account.sip.domain.DomainInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(self, limit=None, page_size=None):
        """
        Asynchronously streams DomainInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.api.v2010.account.sip.domain.DomainInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(page_size=limits["page_size"])

        return await self._version.stream_async(page, limits["limit"])

    def list(self, limit=None, page_size=None):
        """
        Lists DomainInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.api.v2010.account.sip.domain.DomainInstance]
        """
        return list(
            self.stream(
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(self, limit=None, page_size=None):
        """
        Asynchronously lists DomainInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.api.v2010.account.sip.domain.DomainInstance]
        """
        return list(
            await self.stream_async(
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Retrieve a single page of DomainInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return DomainPage(self._version, response, self._solution)

    async def page_async(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Asynchronously retrieve a single page of DomainInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return DomainPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of DomainInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return DomainPage(self._version, response, self._solution)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of DomainInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of DomainInstance
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return DomainPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a DomainContext

        :param sid: The Twilio-provided string that uniquely identifies the SipDomain resource to update.

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainContext
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainContext
        """
        return DomainContext(
            self._version, account_sid=self._solution["account_sid"], sid=sid
        )

    def __call__(self, sid):
        """
        Constructs a DomainContext

        :param sid: The Twilio-provided string that uniquely identifies the SipDomain resource to update.

        :returns: twilio.rest.api.v2010.account.sip.domain.DomainContext
        :rtype: twilio.rest.api.v2010.account.sip.domain.DomainContext
        """
        return DomainContext(
            self._version, account_sid=self._solution["account_sid"], sid=sid
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Api.V2010.DomainList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Events
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page


class SchemaVersionInstance(InstanceResource):
    def __init__(self, version, payload, id: str, schema_version: Optional[int] = None):
        """
        Initialize the SchemaVersionInstance

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """
        super().__init__(version)

        self._properties = {
            "id": payload.get("id"),
            "schema_version": deserialize.integer(payload.get("schema_version")),
            "date_created": deserialize.iso8601_datetime(payload.get("date_created")),
            "url": payload.get("url"),
            "raw": payload.get("raw"),
        }

        self._solution = {
            "id": id,
            "schema_version": schema_version or self._properties["schema_version"],
        }
        self._context: Optional[SchemaVersionContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: SchemaVersionContext for this SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        """
        if self._context is None:
            self._context = SchemaVersionContext(
                self._version,
                id=self._solution["id"],
                schema_version=self._solution["schema_version"],
            )
        return self._context

    @property
    def id(self):
        """
        :returns: The unique identifier of the schema. Each schema can have multiple versions, that share the same id.
        :rtype: str
        """
        return self._properties["id"]

    @property
    def schema_version(self):
        """
        :returns: The version of this schema.
        :rtype: int
        """
        return self._properties["schema_version"]

    @property
    def date_created(self):
        """
        :returns: The date the schema version was created, given in ISO 8601 format.
        :rtype: datetime
        """
        return self._properties["date_created"]

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: str
        """
        return self._properties["url"]

    @property
    def raw(self):
        """
        :returns:
        :rtype: str
        """
        return self._properties["raw"]

    def fetch(self):
        """
        Fetch the SchemaVersionInstance


        :returns: The fetched SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the SchemaVersionInstance


        :returns: The fetched SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """
        return await self._proxy.fetch_async()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Events.V1.SchemaVersionInstance {}>".format(context)


class SchemaVersionContext(InstanceContext):
    def __init__(self, version: Version, id: str, schema_version: int):
        """
        Initialize the SchemaVersionContext

        :param Version version: Version that contains the resource
        :param id: The unique identifier of the schema. Each schema can have multiple versions, that share the same id.
        :param schema_version: The version of the schema

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "id": id,
            "schema_version": schema_version,
        }
        self._uri = "/Schemas/{id}/Versions/{schema_version}".format(**self._solution)

    def fetch(self):
        """
        Fetch the SchemaVersionInstance


        :returns: The fetched SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return SchemaVersionInstance(
            self._version,
            payload,
            id=self._solution["id"],
            schema_version=self._solution["schema_version"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the SchemaVersionInstance


        :returns: The fetched SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return SchemaVersionInstance(
            self._version,
            payload,
            id=self._solution["id"],
            schema_version=self._solution["schema_version"],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Events.V1.SchemaVersionContext {}>".format(context)


class SchemaVersionPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of SchemaVersionInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance
        """
        return SchemaVersionInstance(self._version, payload, id=self._solution["id"])

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Events.V1.SchemaVersionPage>"


class SchemaVersionList(ListResource):
    def __init__(self, version: Version, id: str):
        """
        Initialize the SchemaVersionList

        :param Version version: Version that contains the resource
        :param id: The unique identifier of the schema. Each schema can have multiple versions, that share the same id.

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionList
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionList
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "id": id,
        }
        self._uri = "/Schemas/{id}/Versions".format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams SchemaVersionInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(self, limit=None, page_size=None):
        """
        Asynchronously streams SchemaVersionInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(page_size=limits["page_size"])

        return await self._version.stream_async(page, limits["limit"])

    def list(self, limit=None, page_size=None):
        """
        Lists SchemaVersionInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance]
        """
        return list(
            self.stream(
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(self, limit=None, page_size=None):
        """
        Asynchronously lists SchemaVersionInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.events.v1.schema.schema_version.SchemaVersionInstance]
        """
        return list(
            await self.stream_async(
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Retrieve a single page of SchemaVersionInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return SchemaVersionPage(self._version, response, self._solution)

    async def page_async(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Asynchronously retrieve a single page of SchemaVersionInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return SchemaVersionPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of SchemaVersionInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return SchemaVersionPage(self._version, response, self._solution)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of SchemaVersionInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of SchemaVersionInstance
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return SchemaVersionPage(self._version, response, self._solution)

    def get(self, schema_version):
        """
        Constructs a SchemaVersionContext

        :param schema_version: The version of the schema

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        """
        return SchemaVersionContext(
            self._version, id=self._solution["id"], schema_version=schema_version
        )

    def __call__(self, schema_version):
        """
        Constructs a SchemaVersionContext

        :param schema_version: The version of the schema

        :returns: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        :rtype: twilio.rest.events.v1.schema.schema_version.SchemaVersionContext
        """
        return SchemaVersionContext(
            self._version, id=self._solution["id"], schema_version=schema_version
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Events.V1.SchemaVersionList>"



from warnings import warn

from twilio.rest.frontline_api.FrontlineApiBase import FrontlineApiBase
from twilio.rest.frontline_api.v1.user import UserList


class FrontlineApi(FrontlineApiBase):
    @property
    def users(self) -> UserList:
        warn(
            "users is deprecated. Use v1.users instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return self.v1.users



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Preview
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import deserialize, serialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page
from twilio.rest.preview.hosted_numbers.authorization_document.dependent_hosted_number_order import (
    DependentHostedNumberOrderList,
)


class AuthorizationDocumentInstance(InstanceResource):
    class Status(object):
        OPENED = "opened"
        SIGNING = "signing"
        SIGNED = "signed"
        CANCELED = "canceled"
        FAILED = "failed"

    def __init__(self, version, payload, sid: Optional[str] = None):
        """
        Initialize the AuthorizationDocumentInstance

        :returns: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        super().__init__(version)

        self._properties = {
            "sid": payload.get("sid"),
            "address_sid": payload.get("address_sid"),
            "status": payload.get("status"),
            "email": payload.get("email"),
            "cc_emails": payload.get("cc_emails"),
            "date_created": deserialize.iso8601_datetime(payload.get("date_created")),
            "date_updated": deserialize.iso8601_datetime(payload.get("date_updated")),
            "url": payload.get("url"),
            "links": payload.get("links"),
        }

        self._solution = {
            "sid": sid or self._properties["sid"],
        }
        self._context: Optional[AuthorizationDocumentContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: AuthorizationDocumentContext for this AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentContext
        """
        if self._context is None:
            self._context = AuthorizationDocumentContext(
                self._version,
                sid=self._solution["sid"],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: A 34 character string that uniquely identifies this AuthorizationDocument.
        :rtype: str
        """
        return self._properties["sid"]

    @property
    def address_sid(self):
        """
        :returns: A 34 character string that uniquely identifies the Address resource that is associated with this AuthorizationDocument.
        :rtype: str
        """
        return self._properties["address_sid"]

    @property
    def status(self):
        """
        :returns:
        :rtype: AuthorizationDocumentInstance.Status
        """
        return self._properties["status"]

    @property
    def email(self):
        """
        :returns: Email that this AuthorizationDocument will be sent to for signing.
        :rtype: str
        """
        return self._properties["email"]

    @property
    def cc_emails(self):
        """
        :returns: Email recipients who will be informed when an Authorization Document has been sent and signed.
        :rtype: list[str]
        """
        return self._properties["cc_emails"]

    @property
    def date_created(self):
        """
        :returns: The date this resource was created, given as [GMT RFC 2822](http://www.ietf.org/rfc/rfc2822.txt) format.
        :rtype: datetime
        """
        return self._properties["date_created"]

    @property
    def date_updated(self):
        """
        :returns: The date that this resource was updated, given as [GMT RFC 2822](http://www.ietf.org/rfc/rfc2822.txt) format.
        :rtype: datetime
        """
        return self._properties["date_updated"]

    @property
    def url(self):
        """
        :returns:
        :rtype: str
        """
        return self._properties["url"]

    @property
    def links(self):
        """
        :returns:
        :rtype: dict
        """
        return self._properties["links"]

    def fetch(self):
        """
        Fetch the AuthorizationDocumentInstance


        :returns: The fetched AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the AuthorizationDocumentInstance


        :returns: The fetched AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        return await self._proxy.fetch_async()

    def update(
        self,
        hosted_number_order_sids=values.unset,
        address_sid=values.unset,
        email=values.unset,
        cc_emails=values.unset,
        status=values.unset,
        contact_title=values.unset,
        contact_phone_number=values.unset,
    ):
        """
        Update the AuthorizationDocumentInstance

        :param list[str] hosted_number_order_sids: A list of HostedNumberOrder sids that this AuthorizationDocument will authorize for hosting phone number capabilities on Twilio's platform.
        :param str address_sid: A 34 character string that uniquely identifies the Address resource that is associated with this AuthorizationDocument.
        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param list[str] cc_emails: Email recipients who will be informed when an Authorization Document has been sent and signed
        :param AuthorizationDocumentInstance.Status status:
        :param str contact_title: The title of the person authorized to sign the Authorization Document for this phone number.
        :param str contact_phone_number: The contact phone number of the person authorized to sign the Authorization Document.

        :returns: The updated AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        return self._proxy.update(
            hosted_number_order_sids=hosted_number_order_sids,
            address_sid=address_sid,
            email=email,
            cc_emails=cc_emails,
            status=status,
            contact_title=contact_title,
            contact_phone_number=contact_phone_number,
        )

    async def update_async(
        self,
        hosted_number_order_sids=values.unset,
        address_sid=values.unset,
        email=values.unset,
        cc_emails=values.unset,
        status=values.unset,
        contact_title=values.unset,
        contact_phone_number=values.unset,
    ):
        """
        Asynchronous coroutine to update the AuthorizationDocumentInstance

        :param list[str] hosted_number_order_sids: A list of HostedNumberOrder sids that this AuthorizationDocument will authorize for hosting phone number capabilities on Twilio's platform.
        :param str address_sid: A 34 character string that uniquely identifies the Address resource that is associated with this AuthorizationDocument.
        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param list[str] cc_emails: Email recipients who will be informed when an Authorization Document has been sent and signed
        :param AuthorizationDocumentInstance.Status status:
        :param str contact_title: The title of the person authorized to sign the Authorization Document for this phone number.
        :param str contact_phone_number: The contact phone number of the person authorized to sign the Authorization Document.

        :returns: The updated AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        return await self._proxy.update_async(
            hosted_number_order_sids=hosted_number_order_sids,
            address_sid=address_sid,
            email=email,
            cc_emails=cc_emails,
            status=status,
            contact_title=contact_title,
            contact_phone_number=contact_phone_number,
        )

    @property
    def dependent_hosted_number_orders(self):
        """
        Access the dependent_hosted_number_orders

        :returns: twilio.rest.preview.hosted_numbers.authorization_document.DependentHostedNumberOrderList
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.DependentHostedNumberOrderList
        """
        return self._proxy.dependent_hosted_number_orders

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Preview.HostedNumbers.AuthorizationDocumentInstance {}>".format(
            context
        )


class AuthorizationDocumentContext(InstanceContext):
    def __init__(self, version: Version, sid: str):
        """
        Initialize the AuthorizationDocumentContext

        :param Version version: Version that contains the resource
        :param sid:

        :returns: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentContext
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "sid": sid,
        }
        self._uri = "/AuthorizationDocuments/{sid}".format(**self._solution)

        self._dependent_hosted_number_orders: Optional[
            DependentHostedNumberOrderList
        ] = None

    def fetch(self):
        """
        Fetch the AuthorizationDocumentInstance


        :returns: The fetched AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return AuthorizationDocumentInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the AuthorizationDocumentInstance


        :returns: The fetched AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return AuthorizationDocumentInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    def update(
        self,
        hosted_number_order_sids=values.unset,
        address_sid=values.unset,
        email=values.unset,
        cc_emails=values.unset,
        status=values.unset,
        contact_title=values.unset,
        contact_phone_number=values.unset,
    ):
        """
        Update the AuthorizationDocumentInstance

        :param list[str] hosted_number_order_sids: A list of HostedNumberOrder sids that this AuthorizationDocument will authorize for hosting phone number capabilities on Twilio's platform.
        :param str address_sid: A 34 character string that uniquely identifies the Address resource that is associated with this AuthorizationDocument.
        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param list[str] cc_emails: Email recipients who will be informed when an Authorization Document has been sent and signed
        :param AuthorizationDocumentInstance.Status status:
        :param str contact_title: The title of the person authorized to sign the Authorization Document for this phone number.
        :param str contact_phone_number: The contact phone number of the person authorized to sign the Authorization Document.

        :returns: The updated AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        data = values.of(
            {
                "HostedNumberOrderSids": serialize.map(
                    hosted_number_order_sids, lambda e: e
                ),
                "AddressSid": address_sid,
                "Email": email,
                "CcEmails": serialize.map(cc_emails, lambda e: e),
                "Status": status,
                "ContactTitle": contact_title,
                "ContactPhoneNumber": contact_phone_number,
            }
        )

        payload = self._version.update(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return AuthorizationDocumentInstance(
            self._version, payload, sid=self._solution["sid"]
        )

    async def update_async(
        self,
        hosted_number_order_sids=values.unset,
        address_sid=values.unset,
        email=values.unset,
        cc_emails=values.unset,
        status=values.unset,
        contact_title=values.unset,
        contact_phone_number=values.unset,
    ):
        """
        Asynchronous coroutine to update the AuthorizationDocumentInstance

        :param list[str] hosted_number_order_sids: A list of HostedNumberOrder sids that this AuthorizationDocument will authorize for hosting phone number capabilities on Twilio's platform.
        :param str address_sid: A 34 character string that uniquely identifies the Address resource that is associated with this AuthorizationDocument.
        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param list[str] cc_emails: Email recipients who will be informed when an Authorization Document has been sent and signed
        :param AuthorizationDocumentInstance.Status status:
        :param str contact_title: The title of the person authorized to sign the Authorization Document for this phone number.
        :param str contact_phone_number: The contact phone number of the person authorized to sign the Authorization Document.

        :returns: The updated AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        data = values.of(
            {
                "HostedNumberOrderSids": serialize.map(
                    hosted_number_order_sids, lambda e: e
                ),
                "AddressSid": address_sid,
                "Email": email,
                "CcEmails": serialize.map(cc_emails, lambda e: e),
                "Status": status,
                "ContactTitle": contact_title,
                "ContactPhoneNumber": contact_phone_number,
            }
        )

        payload = await self._version.update_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return AuthorizationDocumentInstance(
            self._version, payload, sid=self._solution["sid"]
        )

    @property
    def dependent_hosted_number_orders(self):
        """
        Access the dependent_hosted_number_orders

        :returns: twilio.rest.preview.hosted_numbers.authorization_document.DependentHostedNumberOrderList
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.DependentHostedNumberOrderList
        """
        if self._dependent_hosted_number_orders is None:
            self._dependent_hosted_number_orders = DependentHostedNumberOrderList(
                self._version,
                self._solution["sid"],
            )
        return self._dependent_hosted_number_orders

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Preview.HostedNumbers.AuthorizationDocumentContext {}>".format(
            context
        )


class AuthorizationDocumentPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of AuthorizationDocumentInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        return AuthorizationDocumentInstance(self._version, payload)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Preview.HostedNumbers.AuthorizationDocumentPage>"


class AuthorizationDocumentList(ListResource):
    def __init__(self, version: Version):
        """
        Initialize the AuthorizationDocumentList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentList
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentList
        """
        super().__init__(version)

        self._uri = "/AuthorizationDocuments"

    def create(
        self,
        hosted_number_order_sids,
        address_sid,
        email,
        contact_title,
        contact_phone_number,
        cc_emails=values.unset,
    ):
        """
        Create the AuthorizationDocumentInstance

        :param list[str] hosted_number_order_sids: A list of HostedNumberOrder sids that this AuthorizationDocument will authorize for hosting phone number capabilities on Twilio's platform.
        :param str address_sid: A 34 character string that uniquely identifies the Address resource that is associated with this AuthorizationDocument.
        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param str contact_title: The title of the person authorized to sign the Authorization Document for this phone number.
        :param str contact_phone_number: The contact phone number of the person authorized to sign the Authorization Document.
        :param list[str] cc_emails: Email recipients who will be informed when an Authorization Document has been sent and signed.

        :returns: The created AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        data = values.of(
            {
                "HostedNumberOrderSids": serialize.map(
                    hosted_number_order_sids, lambda e: e
                ),
                "AddressSid": address_sid,
                "Email": email,
                "ContactTitle": contact_title,
                "ContactPhoneNumber": contact_phone_number,
                "CcEmails": serialize.map(cc_emails, lambda e: e),
            }
        )

        payload = self._version.create(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return AuthorizationDocumentInstance(self._version, payload)

    async def create_async(
        self,
        hosted_number_order_sids,
        address_sid,
        email,
        contact_title,
        contact_phone_number,
        cc_emails=values.unset,
    ):
        """
        Asynchronously create the AuthorizationDocumentInstance

        :param list[str] hosted_number_order_sids: A list of HostedNumberOrder sids that this AuthorizationDocument will authorize for hosting phone number capabilities on Twilio's platform.
        :param str address_sid: A 34 character string that uniquely identifies the Address resource that is associated with this AuthorizationDocument.
        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param str contact_title: The title of the person authorized to sign the Authorization Document for this phone number.
        :param str contact_phone_number: The contact phone number of the person authorized to sign the Authorization Document.
        :param list[str] cc_emails: Email recipients who will be informed when an Authorization Document has been sent and signed.

        :returns: The created AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance
        """
        data = values.of(
            {
                "HostedNumberOrderSids": serialize.map(
                    hosted_number_order_sids, lambda e: e
                ),
                "AddressSid": address_sid,
                "Email": email,
                "ContactTitle": contact_title,
                "ContactPhoneNumber": contact_phone_number,
                "CcEmails": serialize.map(cc_emails, lambda e: e),
            }
        )

        payload = await self._version.create_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return AuthorizationDocumentInstance(self._version, payload)

    def stream(
        self, email=values.unset, status=values.unset, limit=None, page_size=None
    ):
        """
        Streams AuthorizationDocumentInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param AuthorizationDocumentInstance.Status status: Status of an instance resource. It can hold one of the values: 1. opened 2. signing, 3. signed LOA, 4. canceled, 5. failed. See the section entitled [Status Values](https://www.twilio.com/docs/api/phone-numbers/hosted-number-authorization-documents#status-values) for more information on each of these statuses.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(email=email, status=status, page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(
        self, email=values.unset, status=values.unset, limit=None, page_size=None
    ):
        """
        Asynchronously streams AuthorizationDocumentInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param AuthorizationDocumentInstance.Status status: Status of an instance resource. It can hold one of the values: 1. opened 2. signing, 3. signed LOA, 4. canceled, 5. failed. See the section entitled [Status Values](https://www.twilio.com/docs/api/phone-numbers/hosted-number-authorization-documents#status-values) for more information on each of these statuses.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(
            email=email, status=status, page_size=limits["page_size"]
        )

        return await self._version.stream_async(page, limits["limit"])

    def list(self, email=values.unset, status=values.unset, limit=None, page_size=None):
        """
        Lists AuthorizationDocumentInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param AuthorizationDocumentInstance.Status status: Status of an instance resource. It can hold one of the values: 1. opened 2. signing, 3. signed LOA, 4. canceled, 5. failed. See the section entitled [Status Values](https://www.twilio.com/docs/api/phone-numbers/hosted-number-authorization-documents#status-values) for more information on each of these statuses.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance]
        """
        return list(
            self.stream(
                email=email,
                status=status,
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(
        self, email=values.unset, status=values.unset, limit=None, page_size=None
    ):
        """
        Asynchronously lists AuthorizationDocumentInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param AuthorizationDocumentInstance.Status status: Status of an instance resource. It can hold one of the values: 1. opened 2. signing, 3. signed LOA, 4. canceled, 5. failed. See the section entitled [Status Values](https://www.twilio.com/docs/api/phone-numbers/hosted-number-authorization-documents#status-values) for more information on each of these statuses.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentInstance]
        """
        return list(
            await self.stream_async(
                email=email,
                status=status,
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self,
        email=values.unset,
        status=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Retrieve a single page of AuthorizationDocumentInstance records from the API.
        Request is executed immediately

        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param AuthorizationDocumentInstance.Status status: Status of an instance resource. It can hold one of the values: 1. opened 2. signing, 3. signed LOA, 4. canceled, 5. failed. See the section entitled [Status Values](https://www.twilio.com/docs/api/phone-numbers/hosted-number-authorization-documents#status-values) for more information on each of these statuses.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentPage
        """
        data = values.of(
            {
                "Email": email,
                "Status": status,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return AuthorizationDocumentPage(self._version, response)

    async def page_async(
        self,
        email=values.unset,
        status=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Asynchronously retrieve a single page of AuthorizationDocumentInstance records from the API.
        Request is executed immediately

        :param str email: Email that this AuthorizationDocument will be sent to for signing.
        :param AuthorizationDocumentInstance.Status status: Status of an instance resource. It can hold one of the values: 1. opened 2. signing, 3. signed LOA, 4. canceled, 5. failed. See the section entitled [Status Values](https://www.twilio.com/docs/api/phone-numbers/hosted-number-authorization-documents#status-values) for more information on each of these statuses.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentPage
        """
        data = values.of(
            {
                "Email": email,
                "Status": status,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return AuthorizationDocumentPage(self._version, response)

    def get_page(self, target_url):
        """
        Retrieve a specific page of AuthorizationDocumentInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return AuthorizationDocumentPage(self._version, response)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of AuthorizationDocumentInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of AuthorizationDocumentInstance
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return AuthorizationDocumentPage(self._version, response)

    def get(self, sid):
        """
        Constructs a AuthorizationDocumentContext

        :param sid:

        :returns: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentContext
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentContext
        """
        return AuthorizationDocumentContext(self._version, sid=sid)

    def __call__(self, sid):
        """
        Constructs a AuthorizationDocumentContext

        :param sid:

        :returns: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentContext
        :rtype: twilio.rest.preview.hosted_numbers.authorization_document.AuthorizationDocumentContext
        """
        return AuthorizationDocumentContext(self._version, sid=sid)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Preview.HostedNumbers.AuthorizationDocumentList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Microvisor
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version


class AppManifestInstance(InstanceResource):
    def __init__(self, version, payload, app_sid: str):
        """
        Initialize the AppManifestInstance

        :returns: twilio.rest.microvisor.v1.app.app_manifest.AppManifestInstance
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestInstance
        """
        super().__init__(version)

        self._properties = {
            "app_sid": payload.get("app_sid"),
            "hash": payload.get("hash"),
            "encoded_bytes": payload.get("encoded_bytes"),
            "url": payload.get("url"),
        }

        self._solution = {
            "app_sid": app_sid,
        }
        self._context: Optional[AppManifestContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: AppManifestContext for this AppManifestInstance
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestContext
        """
        if self._context is None:
            self._context = AppManifestContext(
                self._version,
                app_sid=self._solution["app_sid"],
            )
        return self._context

    @property
    def app_sid(self):
        """
        :returns: A 34-character string that uniquely identifies this App.
        :rtype: str
        """
        return self._properties["app_sid"]

    @property
    def hash(self):
        """
        :returns: App manifest hash represented as `hash_algorithm:hash_value`.
        :rtype: str
        """
        return self._properties["hash"]

    @property
    def encoded_bytes(self):
        """
        :returns: The base-64 encoded manifest
        :rtype: str
        """
        return self._properties["encoded_bytes"]

    @property
    def url(self):
        """
        :returns: The absolute URL of this Manifest.
        :rtype: str
        """
        return self._properties["url"]

    def fetch(self):
        """
        Fetch the AppManifestInstance


        :returns: The fetched AppManifestInstance
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the AppManifestInstance


        :returns: The fetched AppManifestInstance
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestInstance
        """
        return await self._proxy.fetch_async()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Microvisor.V1.AppManifestInstance {}>".format(context)


class AppManifestContext(InstanceContext):
    def __init__(self, version: Version, app_sid: str):
        """
        Initialize the AppManifestContext

        :param Version version: Version that contains the resource
        :param app_sid: A 34-character string that uniquely identifies this App.

        :returns: twilio.rest.microvisor.v1.app.app_manifest.AppManifestContext
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "app_sid": app_sid,
        }
        self._uri = "/Apps/{app_sid}/Manifest".format(**self._solution)

    def fetch(self):
        """
        Fetch the AppManifestInstance


        :returns: The fetched AppManifestInstance
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return AppManifestInstance(
            self._version,
            payload,
            app_sid=self._solution["app_sid"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the AppManifestInstance


        :returns: The fetched AppManifestInstance
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return AppManifestInstance(
            self._version,
            payload,
            app_sid=self._solution["app_sid"],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Microvisor.V1.AppManifestContext {}>".format(context)


class AppManifestList(ListResource):
    def __init__(self, version: Version, app_sid: str):
        """
        Initialize the AppManifestList

        :param Version version: Version that contains the resource
        :param app_sid: A 34-character string that uniquely identifies this App.

        :returns: twilio.rest.microvisor.v1.app.app_manifest.AppManifestList
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestList
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "app_sid": app_sid,
        }

    def get(self):
        """
        Constructs a AppManifestContext


        :returns: twilio.rest.microvisor.v1.app.app_manifest.AppManifestContext
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestContext
        """
        return AppManifestContext(self._version, app_sid=self._solution["app_sid"])

    def __call__(self):
        """
        Constructs a AppManifestContext


        :returns: twilio.rest.microvisor.v1.app.app_manifest.AppManifestContext
        :rtype: twilio.rest.microvisor.v1.app.app_manifest.AppManifestContext
        """
        return AppManifestContext(self._version, app_sid=self._solution["app_sid"])

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Microvisor.V1.AppManifestList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Sync
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page


class DocumentPermissionInstance(InstanceResource):
    def __init__(
        self,
        version,
        payload,
        service_sid: str,
        document_sid: str,
        identity: Optional[str] = None,
    ):
        """
        Initialize the DocumentPermissionInstance

        :returns: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """
        super().__init__(version)

        self._properties = {
            "account_sid": payload.get("account_sid"),
            "service_sid": payload.get("service_sid"),
            "document_sid": payload.get("document_sid"),
            "identity": payload.get("identity"),
            "read": payload.get("read"),
            "write": payload.get("write"),
            "manage": payload.get("manage"),
            "url": payload.get("url"),
        }

        self._solution = {
            "service_sid": service_sid,
            "document_sid": document_sid,
            "identity": identity or self._properties["identity"],
        }
        self._context: Optional[DocumentPermissionContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: DocumentPermissionContext for this DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionContext
        """
        if self._context is None:
            self._context = DocumentPermissionContext(
                self._version,
                service_sid=self._solution["service_sid"],
                document_sid=self._solution["document_sid"],
                identity=self._solution["identity"],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the Document Permission resource.
        :rtype: str
        """
        return self._properties["account_sid"]

    @property
    def service_sid(self):
        """
        :returns: The SID of the [Sync Service](https://www.twilio.com/docs/sync/api/service) the resource is associated with.
        :rtype: str
        """
        return self._properties["service_sid"]

    @property
    def document_sid(self):
        """
        :returns: The SID of the Sync Document to which the Document Permission applies.
        :rtype: str
        """
        return self._properties["document_sid"]

    @property
    def identity(self):
        """
        :returns: The application-defined string that uniquely identifies the resource's User within the Service to an FPA token.
        :rtype: str
        """
        return self._properties["identity"]

    @property
    def read(self):
        """
        :returns: Whether the identity can read the Sync Document.
        :rtype: bool
        """
        return self._properties["read"]

    @property
    def write(self):
        """
        :returns: Whether the identity can update the Sync Document.
        :rtype: bool
        """
        return self._properties["write"]

    @property
    def manage(self):
        """
        :returns: Whether the identity can delete the Sync Document.
        :rtype: bool
        """
        return self._properties["manage"]

    @property
    def url(self):
        """
        :returns: The absolute URL of the Sync Document Permission resource.
        :rtype: str
        """
        return self._properties["url"]

    def delete(self):
        """
        Deletes the DocumentPermissionInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    async def delete_async(self):
        """
        Asynchronous coroutine that deletes the DocumentPermissionInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return await self._proxy.delete_async()

    def fetch(self):
        """
        Fetch the DocumentPermissionInstance


        :returns: The fetched DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the DocumentPermissionInstance


        :returns: The fetched DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """
        return await self._proxy.fetch_async()

    def update(self, read, write, manage):
        """
        Update the DocumentPermissionInstance

        :param bool read: Whether the identity can read the Sync Document. Default value is `false`.
        :param bool write: Whether the identity can update the Sync Document. Default value is `false`.
        :param bool manage: Whether the identity can delete the Sync Document. Default value is `false`.

        :returns: The updated DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """
        return self._proxy.update(
            read=read,
            write=write,
            manage=manage,
        )

    async def update_async(self, read, write, manage):
        """
        Asynchronous coroutine to update the DocumentPermissionInstance

        :param bool read: Whether the identity can read the Sync Document. Default value is `false`.
        :param bool write: Whether the identity can update the Sync Document. Default value is `false`.
        :param bool manage: Whether the identity can delete the Sync Document. Default value is `false`.

        :returns: The updated DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """
        return await self._proxy.update_async(
            read=read,
            write=write,
            manage=manage,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Sync.V1.DocumentPermissionInstance {}>".format(context)


class DocumentPermissionContext(InstanceContext):
    def __init__(
        self, version: Version, service_sid: str, document_sid: str, identity: str
    ):
        """
        Initialize the DocumentPermissionContext

        :param Version version: Version that contains the resource
        :param service_sid: The SID of the [Sync Service](https://www.twilio.com/docs/sync/api/service) with the Document Permission resource to update.
        :param document_sid: The SID of the Sync Document with the Document Permission resource to update. Can be the Document resource's `sid` or its `unique_name`.
        :param identity: The application-defined string that uniquely identifies the User's Document Permission resource to update.

        :returns: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionContext
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "service_sid": service_sid,
            "document_sid": document_sid,
            "identity": identity,
        }
        self._uri = "/Services/{service_sid}/Documents/{document_sid}/Permissions/{identity}".format(
            **self._solution
        )

    def delete(self):
        """
        Deletes the DocumentPermissionInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(
            method="DELETE",
            uri=self._uri,
        )

    async def delete_async(self):
        """
        Asynchronous coroutine that deletes the DocumentPermissionInstance


        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return await self._version.delete_async(
            method="DELETE",
            uri=self._uri,
        )

    def fetch(self):
        """
        Fetch the DocumentPermissionInstance


        :returns: The fetched DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return DocumentPermissionInstance(
            self._version,
            payload,
            service_sid=self._solution["service_sid"],
            document_sid=self._solution["document_sid"],
            identity=self._solution["identity"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the DocumentPermissionInstance


        :returns: The fetched DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return DocumentPermissionInstance(
            self._version,
            payload,
            service_sid=self._solution["service_sid"],
            document_sid=self._solution["document_sid"],
            identity=self._solution["identity"],
        )

    def update(self, read, write, manage):
        """
        Update the DocumentPermissionInstance

        :param bool read: Whether the identity can read the Sync Document. Default value is `false`.
        :param bool write: Whether the identity can update the Sync Document. Default value is `false`.
        :param bool manage: Whether the identity can delete the Sync Document. Default value is `false`.

        :returns: The updated DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """
        data = values.of(
            {
                "Read": read,
                "Write": write,
                "Manage": manage,
            }
        )

        payload = self._version.update(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return DocumentPermissionInstance(
            self._version,
            payload,
            service_sid=self._solution["service_sid"],
            document_sid=self._solution["document_sid"],
            identity=self._solution["identity"],
        )

    async def update_async(self, read, write, manage):
        """
        Asynchronous coroutine to update the DocumentPermissionInstance

        :param bool read: Whether the identity can read the Sync Document. Default value is `false`.
        :param bool write: Whether the identity can update the Sync Document. Default value is `false`.
        :param bool manage: Whether the identity can delete the Sync Document. Default value is `false`.

        :returns: The updated DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """
        data = values.of(
            {
                "Read": read,
                "Write": write,
                "Manage": manage,
            }
        )

        payload = await self._version.update_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return DocumentPermissionInstance(
            self._version,
            payload,
            service_sid=self._solution["service_sid"],
            document_sid=self._solution["document_sid"],
            identity=self._solution["identity"],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Sync.V1.DocumentPermissionContext {}>".format(context)


class DocumentPermissionPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of DocumentPermissionInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance
        """
        return DocumentPermissionInstance(
            self._version,
            payload,
            service_sid=self._solution["service_sid"],
            document_sid=self._solution["document_sid"],
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Sync.V1.DocumentPermissionPage>"


class DocumentPermissionList(ListResource):
    def __init__(self, version: Version, service_sid: str, document_sid: str):
        """
        Initialize the DocumentPermissionList

        :param Version version: Version that contains the resource
        :param service_sid: The SID of the [Sync Service](https://www.twilio.com/docs/sync/api/service) with the Document Permission resources to read.
        :param document_sid: The SID of the Sync Document with the Document Permission resources to read. Can be the Document resource's `sid` or its `unique_name`.

        :returns: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionList
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionList
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "service_sid": service_sid,
            "document_sid": document_sid,
        }
        self._uri = (
            "/Services/{service_sid}/Documents/{document_sid}/Permissions".format(
                **self._solution
            )
        )

    def stream(self, limit=None, page_size=None):
        """
        Streams DocumentPermissionInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(self, limit=None, page_size=None):
        """
        Asynchronously streams DocumentPermissionInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(page_size=limits["page_size"])

        return await self._version.stream_async(page, limits["limit"])

    def list(self, limit=None, page_size=None):
        """
        Lists DocumentPermissionInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance]
        """
        return list(
            self.stream(
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(self, limit=None, page_size=None):
        """
        Asynchronously lists DocumentPermissionInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionInstance]
        """
        return list(
            await self.stream_async(
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Retrieve a single page of DocumentPermissionInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return DocumentPermissionPage(self._version, response, self._solution)

    async def page_async(
        self, page_token=values.unset, page_number=values.unset, page_size=values.unset
    ):
        """
        Asynchronously retrieve a single page of DocumentPermissionInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionPage
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return DocumentPermissionPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of DocumentPermissionInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return DocumentPermissionPage(self._version, response, self._solution)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of DocumentPermissionInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of DocumentPermissionInstance
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return DocumentPermissionPage(self._version, response, self._solution)

    def get(self, identity):
        """
        Constructs a DocumentPermissionContext

        :param identity: The application-defined string that uniquely identifies the User's Document Permission resource to update.

        :returns: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionContext
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionContext
        """
        return DocumentPermissionContext(
            self._version,
            service_sid=self._solution["service_sid"],
            document_sid=self._solution["document_sid"],
            identity=identity,
        )

    def __call__(self, identity):
        """
        Constructs a DocumentPermissionContext

        :param identity: The application-defined string that uniquely identifies the User's Document Permission resource to update.

        :returns: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionContext
        :rtype: twilio.rest.sync.v1.service.document.document_permission.DocumentPermissionContext
        """
        return DocumentPermissionContext(
            self._version,
            service_sid=self._solution["service_sid"],
            document_sid=self._solution["document_sid"],
            identity=identity,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Sync.V1.DocumentPermissionList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Media
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""


from typing import Optional
from twilio.base import deserialize, serialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page


class MediaProcessorInstance(InstanceResource):
    class Order(object):
        ASC = "asc"
        DESC = "desc"

    class Status(object):
        FAILED = "failed"
        STARTED = "started"
        ENDED = "ended"

    def __init__(self, version, payload, sid: Optional[str] = None):
        """
        Initialize the MediaProcessorInstance

        :returns: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        super().__init__(version)

        self._properties = {
            "account_sid": payload.get("account_sid"),
            "sid": payload.get("sid"),
            "date_created": deserialize.iso8601_datetime(payload.get("date_created")),
            "date_updated": deserialize.iso8601_datetime(payload.get("date_updated")),
            "extension": payload.get("extension"),
            "extension_context": payload.get("extension_context"),
            "status": payload.get("status"),
            "url": payload.get("url"),
            "ended_reason": payload.get("ended_reason"),
            "status_callback": payload.get("status_callback"),
            "status_callback_method": payload.get("status_callback_method"),
            "max_duration": deserialize.integer(payload.get("max_duration")),
        }

        self._solution = {
            "sid": sid or self._properties["sid"],
        }
        self._context: Optional[MediaProcessorContext] = None

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: MediaProcessorContext for this MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorContext
        """
        if self._context is None:
            self._context = MediaProcessorContext(
                self._version,
                sid=self._solution["sid"],
            )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the [Account](https://www.twilio.com/docs/iam/api/account) that created the MediaProcessor resource.
        :rtype: str
        """
        return self._properties["account_sid"]

    @property
    def sid(self):
        """
        :returns: The unique string generated to identify the MediaProcessor resource.
        :rtype: str
        """
        return self._properties["sid"]

    @property
    def date_created(self):
        """
        :returns: The date and time in GMT when the resource was created specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_created"]

    @property
    def date_updated(self):
        """
        :returns: The date and time in GMT when the resource was last updated specified in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format.
        :rtype: datetime
        """
        return self._properties["date_updated"]

    @property
    def extension(self):
        """
        :returns: The [Media Extension](/docs/live/api/media-extensions-overview) name or URL. Ex: `video-composer-v2`
        :rtype: str
        """
        return self._properties["extension"]

    @property
    def extension_context(self):
        """
        :returns: The context of the Media Extension, represented as a JSON dictionary. See the documentation for the specific [Media Extension](/docs/live/api/media-extensions-overview) you are using for more information about the context to send.
        :rtype: str
        """
        return self._properties["extension_context"]

    @property
    def status(self):
        """
        :returns:
        :rtype: MediaProcessorInstance.Status
        """
        return self._properties["status"]

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource.
        :rtype: str
        """
        return self._properties["url"]

    @property
    def ended_reason(self):
        """
        :returns: The reason why a MediaProcessor ended. When a MediaProcessor is in progress, will be `null`. When a MediaProcessor is completed, can be `ended-via-api`, `max-duration-exceeded`, `error-loading-extension`, `error-streaming-media` or `internal-service-error`. See [ended reasons](/docs/live/api/mediaprocessors#mediaprocessor-ended-reason-values) for more details.
        :rtype: str
        """
        return self._properties["ended_reason"]

    @property
    def status_callback(self):
        """
        :returns: The URL to which Twilio will send asynchronous webhook requests for every MediaProcessor event. See [Status Callbacks](/docs/live/status-callbacks) for details.
        :rtype: str
        """
        return self._properties["status_callback"]

    @property
    def status_callback_method(self):
        """
        :returns: The HTTP method Twilio should use to call the `status_callback` URL. Can be `POST` or `GET` and the default is `POST`.
        :rtype: str
        """
        return self._properties["status_callback_method"]

    @property
    def max_duration(self):
        """
        :returns: The maximum time, in seconds, that the MediaProcessor can run before automatically ends. The default value is 300 seconds, and the maximum value is 90000 seconds. Once this maximum duration is reached, Twilio will end the MediaProcessor, regardless of whether media is still streaming.
        :rtype: int
        """
        return self._properties["max_duration"]

    def fetch(self):
        """
        Fetch the MediaProcessorInstance


        :returns: The fetched MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the MediaProcessorInstance


        :returns: The fetched MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        return await self._proxy.fetch_async()

    def update(self, status):
        """
        Update the MediaProcessorInstance

        :param MediaProcessorInstance.UpdateStatus status:

        :returns: The updated MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        return self._proxy.update(
            status=status,
        )

    async def update_async(self, status):
        """
        Asynchronous coroutine to update the MediaProcessorInstance

        :param MediaProcessorInstance.UpdateStatus status:

        :returns: The updated MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        return await self._proxy.update_async(
            status=status,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Media.V1.MediaProcessorInstance {}>".format(context)


class MediaProcessorContext(InstanceContext):
    def __init__(self, version: Version, sid: str):
        """
        Initialize the MediaProcessorContext

        :param Version version: Version that contains the resource
        :param sid: The SID of the MediaProcessor resource to update.

        :returns: twilio.rest.media.v1.media_processor.MediaProcessorContext
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorContext
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "sid": sid,
        }
        self._uri = "/MediaProcessors/{sid}".format(**self._solution)

    def fetch(self):
        """
        Fetch the MediaProcessorInstance


        :returns: The fetched MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return MediaProcessorInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    async def fetch_async(self):
        """
        Asynchronous coroutine to fetch the MediaProcessorInstance


        :returns: The fetched MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return MediaProcessorInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    def update(self, status):
        """
        Update the MediaProcessorInstance

        :param MediaProcessorInstance.UpdateStatus status:

        :returns: The updated MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        data = values.of(
            {
                "Status": status,
            }
        )

        payload = self._version.update(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return MediaProcessorInstance(self._version, payload, sid=self._solution["sid"])

    async def update_async(self, status):
        """
        Asynchronous coroutine to update the MediaProcessorInstance

        :param MediaProcessorInstance.UpdateStatus status:

        :returns: The updated MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        data = values.of(
            {
                "Status": status,
            }
        )

        payload = await self._version.update_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return MediaProcessorInstance(self._version, payload, sid=self._solution["sid"])

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Media.V1.MediaProcessorContext {}>".format(context)


class MediaProcessorPage(Page):
    def get_instance(self, payload):
        """
        Build an instance of MediaProcessorInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        return MediaProcessorInstance(self._version, payload)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Media.V1.MediaProcessorPage>"


class MediaProcessorList(ListResource):
    def __init__(self, version: Version):
        """
        Initialize the MediaProcessorList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.media.v1.media_processor.MediaProcessorList
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorList
        """
        super().__init__(version)

        self._uri = "/MediaProcessors"

    def create(
        self,
        extension,
        extension_context,
        extension_environment=values.unset,
        status_callback=values.unset,
        status_callback_method=values.unset,
        max_duration=values.unset,
    ):
        """
        Create the MediaProcessorInstance

        :param str extension: The [Media Extension](/docs/live/api/media-extensions-overview) name or URL. Ex: `video-composer-v2`
        :param str extension_context: The context of the Media Extension, represented as a JSON dictionary. See the documentation for the specific [Media Extension](/docs/live/api/media-extensions-overview) you are using for more information about the context to send.
        :param object extension_environment: User-defined environment variables for the Media Extension, represented as a JSON dictionary of key/value strings. See the documentation for the specific [Media Extension](/docs/live/api/media-extensions-overview) you are using for more information about whether you need to provide this.
        :param str status_callback: The URL to which Twilio will send asynchronous webhook requests for every MediaProcessor event. See [Status Callbacks](/docs/live/status-callbacks) for details.
        :param str status_callback_method: The HTTP method Twilio should use to call the `status_callback` URL. Can be `POST` or `GET` and the default is `POST`.
        :param int max_duration: The maximum time, in seconds, that the MediaProcessor can run before automatically ends. The default value is 300 seconds, and the maximum value is 90000 seconds. Once this maximum duration is reached, Twilio will end the MediaProcessor, regardless of whether media is still streaming.

        :returns: The created MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        data = values.of(
            {
                "Extension": extension,
                "ExtensionContext": extension_context,
                "ExtensionEnvironment": serialize.object(extension_environment),
                "StatusCallback": status_callback,
                "StatusCallbackMethod": status_callback_method,
                "MaxDuration": max_duration,
            }
        )

        payload = self._version.create(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return MediaProcessorInstance(self._version, payload)

    async def create_async(
        self,
        extension,
        extension_context,
        extension_environment=values.unset,
        status_callback=values.unset,
        status_callback_method=values.unset,
        max_duration=values.unset,
    ):
        """
        Asynchronously create the MediaProcessorInstance

        :param str extension: The [Media Extension](/docs/live/api/media-extensions-overview) name or URL. Ex: `video-composer-v2`
        :param str extension_context: The context of the Media Extension, represented as a JSON dictionary. See the documentation for the specific [Media Extension](/docs/live/api/media-extensions-overview) you are using for more information about the context to send.
        :param object extension_environment: User-defined environment variables for the Media Extension, represented as a JSON dictionary of key/value strings. See the documentation for the specific [Media Extension](/docs/live/api/media-extensions-overview) you are using for more information about whether you need to provide this.
        :param str status_callback: The URL to which Twilio will send asynchronous webhook requests for every MediaProcessor event. See [Status Callbacks](/docs/live/status-callbacks) for details.
        :param str status_callback_method: The HTTP method Twilio should use to call the `status_callback` URL. Can be `POST` or `GET` and the default is `POST`.
        :param int max_duration: The maximum time, in seconds, that the MediaProcessor can run before automatically ends. The default value is 300 seconds, and the maximum value is 90000 seconds. Once this maximum duration is reached, Twilio will end the MediaProcessor, regardless of whether media is still streaming.

        :returns: The created MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorInstance
        """
        data = values.of(
            {
                "Extension": extension,
                "ExtensionContext": extension_context,
                "ExtensionEnvironment": serialize.object(extension_environment),
                "StatusCallback": status_callback,
                "StatusCallbackMethod": status_callback_method,
                "MaxDuration": max_duration,
            }
        )

        payload = await self._version.create_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return MediaProcessorInstance(self._version, payload)

    def stream(
        self, order=values.unset, status=values.unset, limit=None, page_size=None
    ):
        """
        Streams MediaProcessorInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param MediaProcessorInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaProcessorInstance.Status status: Status to filter by, with possible values `started`, `ended` or `failed`.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_processor.MediaProcessorInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(order=order, status=status, page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(
        self, order=values.unset, status=values.unset, limit=None, page_size=None
    ):
        """
        Asynchronously streams MediaProcessorInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param MediaProcessorInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaProcessorInstance.Status status: Status to filter by, with possible values `started`, `ended` or `failed`.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_processor.MediaProcessorInstance]
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(
            order=order, status=status, page_size=limits["page_size"]
        )

        return await self._version.stream_async(page, limits["limit"])

    def list(self, order=values.unset, status=values.unset, limit=None, page_size=None):
        """
        Lists MediaProcessorInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param MediaProcessorInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaProcessorInstance.Status status: Status to filter by, with possible values `started`, `ended` or `failed`.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_processor.MediaProcessorInstance]
        """
        return list(
            self.stream(
                order=order,
                status=status,
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(
        self, order=values.unset, status=values.unset, limit=None, page_size=None
    ):
        """
        Asynchronously lists MediaProcessorInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param MediaProcessorInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaProcessorInstance.Status status: Status to filter by, with possible values `started`, `ended` or `failed`.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_processor.MediaProcessorInstance]
        """
        return list(
            await self.stream_async(
                order=order,
                status=status,
                limit=limit,
                page_size=page_size,
            )
        )

    def page(
        self,
        order=values.unset,
        status=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Retrieve a single page of MediaProcessorInstance records from the API.
        Request is executed immediately

        :param MediaProcessorInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaProcessorInstance.Status status: Status to filter by, with possible values `started`, `ended` or `failed`.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorPage
        """
        data = values.of(
            {
                "Order": order,
                "Status": status,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return MediaProcessorPage(self._version, response)

    async def page_async(
        self,
        order=values.unset,
        status=values.unset,
        page_token=values.unset,
        page_number=values.unset,
        page_size=values.unset,
    ):
        """
        Asynchronously retrieve a single page of MediaProcessorInstance records from the API.
        Request is executed immediately

        :param MediaProcessorInstance.Order order: The sort order of the list by `date_created`. Can be: `asc` (ascending) or `desc` (descending) with `desc` as the default.
        :param MediaProcessorInstance.Status status: Status to filter by, with possible values `started`, `ended` or `failed`.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorPage
        """
        data = values.of(
            {
                "Order": order,
                "Status": status,
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return MediaProcessorPage(self._version, response)

    def get_page(self, target_url):
        """
        Retrieve a specific page of MediaProcessorInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorPage
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return MediaProcessorPage(self._version, response)

    async def get_page_async(self, target_url):
        """
        Asynchronously retrieve a specific page of MediaProcessorInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of MediaProcessorInstance
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorPage
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return MediaProcessorPage(self._version, response)

    def get(self, sid):
        """
        Constructs a MediaProcessorContext

        :param sid: The SID of the MediaProcessor resource to update.

        :returns: twilio.rest.media.v1.media_processor.MediaProcessorContext
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorContext
        """
        return MediaProcessorContext(self._version, sid=sid)

    def __call__(self, sid):
        """
        Constructs a MediaProcessorContext

        :param sid: The SID of the MediaProcessor resource to update.

        :returns: twilio.rest.media.v1.media_processor.MediaProcessorContext
        :rtype: twilio.rest.media.v1.media_processor.MediaProcessorContext
        """
        return MediaProcessorContext(self._version, sid=sid)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return "<Twilio.Media.V1.MediaProcessorList>"



from twilio.jwt.taskrouter import TaskRouterCapabilityToken


class WorkerCapabilityToken(TaskRouterCapabilityToken):
    def __init__(
        self, account_sid, auth_token, workspace_sid, worker_sid, ttl=3600, **kwargs
    ):
        """
        :param kwargs:
            All kwarg parameters supported by TaskRouterCapabilityToken
            :param bool allow_fetch_activities: shortcut to calling allow_fetch_activities,
                                                defaults to True
            :param bool allow_fetch_reservations: shortcut to calling allow_fetch_reservations,
                                                  defaults to True
            :param bool allow_fetch_worker_reservations: shortcut to calling allow_fetch_worker_reservations,
                                                         defaults to True
            :param bool allow_update_activities: shortcut to calling allow_update_activities,
                                                 defaults to False
            :param bool allow_update_reservations: shortcut to calling allow_update_reservations,
                                                   defaults to False
        """
        super(WorkerCapabilityToken, self).__init__(
            account_sid=account_sid,
            auth_token=auth_token,
            workspace_sid=workspace_sid,
            channel_id=worker_sid,
            ttl=ttl,
            **kwargs
        )

        if kwargs.get("allow_fetch_activities", True):
            self.allow_fetch_activities()
        if kwargs.get("allow_fetch_reservations", True):
            self.allow_fetch_reservations()
        if kwargs.get("allow_fetch_worker_reservations", True):
            self.allow_fetch_worker_reservations()
        if kwargs.get("allow_update_activities", False):
            self.allow_update_activities()
        if kwargs.get("allow_update_reservations", False):
            self.allow_update_reservations()

    @property
    def resource_url(self):
        return "{}/Workers/{}".format(self.workspace_url, self.channel_id)

    @property
    def channel_prefix(self):
        return "WK"

    def allow_fetch_activities(self):
        self._make_policy(self.workspace_url + "/Activities", "GET", True)

    def allow_fetch_reservations(self):
        self._make_policy(self.workspace_url + "/Tasks/**", "GET", True)

    def allow_fetch_worker_reservations(self):
        self._make_policy(self.resource_url + "/Reservations/**", "GET", True)

    def allow_update_activities(self):
        post_filter = {"ActivitySid": {"required": True}}
        self._make_policy(self.resource_url, "POST", True, post_filter=post_filter)

    def allow_update_reservations(self):
        self._make_policy(self.workspace_url + "/Tasks/**", "POST", True)
        self._make_policy(self.resource_url + "/Reservations/**", "POST", True)

    def __str__(self):
        return "<WorkerCapabilityToken {}>".format(self.to_jwt())


class TaskQueueCapabilityToken(TaskRouterCapabilityToken):
    def __init__(
        self, account_sid, auth_token, workspace_sid, task_queue_sid, ttl=3600, **kwargs
    ):
        super(TaskQueueCapabilityToken, self).__init__(
            account_sid=account_sid,
            auth_token=auth_token,
            workspace_sid=workspace_sid,
            channel_id=task_queue_sid,
            ttl=ttl,
            **kwargs
        )

    @property
    def resource_url(self):
        return "{}/TaskQueues/{}".format(self.workspace_url, self.channel_id)

    @property
    def channel_prefix(self):
        return "WQ"

    def __str__(self):
        return "<TaskQueueCapabilityToken {}>".format(self.to_jwt())


class WorkspaceCapabilityToken(TaskRouterCapabilityToken):
    def __init__(self, account_sid, auth_token, workspace_sid, ttl=3600, **kwargs):
        super(WorkspaceCapabilityToken, self).__init__(
            account_sid=account_sid,
            auth_token=auth_token,
            workspace_sid=workspace_sid,
            channel_id=workspace_sid,
            ttl=ttl,
            **kwargs
        )

    @property
    def resource_url(self):
        return self.workspace_url

    @property
    def channel_prefix(self):
        return "WS"

    def __str__(self):
        return "<WorkspaceCapabilityToken {}>".format(self.to_jwt())
